#!/opt/opensvc/bin/python
""" 
Verify file content. The collector provides the format with
wildcards. The module replace the wildcards with contextual
values.

The variable format is json-serialized:

{
  "path": "/some/path/to/file",
  "check": "some pattern into the file",
  "fmt": "full added content with %%HOSTNAME%%@corp.com",
 - or - 
  "ref": "http:// ..."
}

Wildcards:
%%ENV:VARNAME%%		Any environment variable value
%%HOSTNAME%%		Hostname
%%SHORT_HOSTNAME%%	Short hostname

"""

import os
import sys
import json
import stat
import re
import urllib
import tempfile

sys.path.append(os.path.dirname(__file__))

from comp import *

MAXSZ = 8*1024*1024

class FileInc(object):
    def __init__(self, prefix='OSVC_COMP_FILEINC_'):
        self.prefix = prefix.upper()
        self.files = {}
        self.ok = {}
        self.checks = []
        self.upds = {}

        self.sysname, self.nodename, x, x, self.machine = os.uname()

        for k in [ key for key in os.environ if key.startswith(self.prefix)]:
            try:
                self.add_file(os.environ[k])
            except ValueError:
                print >>sys.stderr, 'key syntax error on var[', k, '] = ',os.environ[k]

        if len(self.checks) == 0:
            print "no appliable variable found in rulesets", self.prefix
            raise NotApplicable()

    def fixable(self):
        return RET_NA

    def get_env_item(self, d, item):
        if item not in d:
            return d
        if type(d[item]) != str and type(d[item]) != unicode:
            return d
        if not d[item].startswith("%%"):
            return d
        s = d[item].strip('%').replace('ENV:', '')
        if not s.startswith('OSVC_COMP_'):
            s = 'OSVC_COMP_'+s
        if s not in os.environ:
            print >>sys.stderr, "%s is not set"%s
            raise
        d[item] = os.environ[s]
        try:
            d[item] = int(d[item])
        except:
            pass
        return d

    def parse_fmt(self, x):
        if isinstance(x, int):
            x = str(x)+'\n'
        x = x.replace('%%HOSTNAME%%', self.nodename)
        x = x.replace('%%SHORT_HOSTNAME%%', self.nodename.split('.')[0])
        if not x.endswith('\n'):
            x += '\n'
        return x

    def parse_ref(self, url):
        f = tempfile.NamedTemporaryFile()
        tmpf = f.name
        try:
            fname, headers = urllib.urlretrieve(url, tmpf)
            if 'invalid file' in headers.values():
                print >>sys.stderr, url, "not found on collector"
                return RET_ERR
            content = unicode(f.read())
            f.close()
        except:
             print >>sys.stderr, url, "not found on collector"
             return ''
        if '<title>404 Not Found</title>' in content:
            print >>sys.stderr, url, "not found on collector"
            return ''
        return self.parse_fmt(content)

    def subst(self, v):
        if type(v) == list:
            l = []
            for _v in v:
                l.append(self.subst(_v))
            return l
        if type(v) != str and type(v) != unicode:
            return v
        p = re.compile('%%ENV:\w+%%')
        for m in p.findall(v):
            s = m.strip("%").replace('ENV:', '')
            if s in os.environ:
                _v = os.environ[s]
            elif 'OSVC_COMP_'+s in os.environ:
                _v = os.environ['OSVC_COMP_'+s]
            else:
                print >>sys.stderr, s, 'is not an env variable'
                raise NotApplicable()
            v = v.replace(m, _v)
        return v

    def read_file(self, path):
        out = ''
        try :
            f = open(path, 'r')
            out = f.read()
            f.close()
        except IOError as (errno, strerror):
            print "cannot read '%s', error=%d - %s" %(path, errno, strerror)
            return RET_ERR, ''
        except:
            print >>sys.stderr, "Cannot open '%s', unexpected error:", sys.exc_info()[0]
            return RET_ERR, ''
        return RET_OK, out

    def add_file(self,v):
        r = RET_OK
        d = json.loads(v)
        if 'path' not in d:
            print >>sys.stderr, "'path' should be defined:", d
            r |= RET_ERR
        if 'fmt' in d and 'ref' in d:
            print >>sys.stderr, "'fmt' and 'ref' are exclusive:", d
            r |= RET_ERR
        try:
            d = self.get_env_item(d, 'check')
        except:
            r |= RET_ERR
        for k in ('path', 'check', 'fmt', 'ref'):
            if k in d:
                d[k] = self.subst(d[k])
        if not d['path'] in self.upds:
            self.upds[d['path']] = 0
        if not d['path'] in self.files:
            try:
                fsz = os.path.getsize(d['path'])
            except:
                fsz = 0
            if fsz > MAXSZ:
                self.ok[d['path']] = 0
                self.files[d['path']] = ''
                print >>sys.stderr, "file '%s' is too large [%.2f Mb] to fit" %(d['path'], fsz/(1024.*1024))
                r |= RET_ERR
            else:
                self.ok[d['path']] = 1
                rx,text = self.read_file(d['path'])
                if rx == RET_OK:
                    self.files[d['path']] = text
                else:
                    self.files[d['path']] = ''
        c = ''
        if 'fmt' in d:
            c = self.parse_fmt(d['fmt'])
        elif 'ref' in d:
            c = self.parse_ref(d['ref'])
        else:
            print >>sys.stderr, "'fmt' or 'ref' should be defined:", d
            r |= RET_ERR
        regex = re.compile(d['check'])
        if len(regex.find(c)) > 0:
            val = True
        else:
            val = False
            r |= RET_ERR
        self.checks.append({'check':d['check'], 'path':d['path'], 'add':c, 'valid':val})
        return r
        
    def check_file(self):
        r = RET_OK
        for ck in self.checks:
             if not ck['valid']:
                 print >>sys.stderr, "Pattern '%s' for '%s' is absent in the requested content" %(ck['check'], ck['path'])
                 r |= RET_ERR
                 continue
             pr = RET_ERR
             lines = self.files[ck['path']].split('\n')
             for line in lines:
                 if ck['check'] in line:
                     pr = RET_OK
                     break
             if pr == RET_OK:
                 print "Pattern '%s' matches '%s' contents, line: %s => OK" %(ck['check'], ck['path'], '"'+line+'"')
             else:
                 print >>sys.stderr, "Pattern '%s' **does not match** '%s' contents => BAD" %(ck['check'], ck['path'])
             r |= pr
        return r

    def check(self):
        r = 0
        r |= self.check_file()
        return r

    def add_2_files(self):
        r = RET_OK
        for path in self.files:
            if self.upds[path] == 0:
                continue
            if self.ok[path] != 1:
                print >>sys.stderr, "skip file '%s': too large to fit => BAD" %path
                r |= RET_ERR
                continue
            if not os.path.exists(path):
                print >>sys.stderr, "'%s' will be created, PLEASE check owner and permissions" %path
            try:
                f = open(path, 'w+')
                f.truncate()
                f.writelines(self.files[path])
                f.close()
                print "'%s' successfully updated => OK" %path
            except:
                print >>sys.stderr, "fail to append to '%s' => BAD" %path
                r |= RET_ERR
        return r

    def fix_files(self):
        r = RET_OK
        for ck in self.checks:
             if not ck['valid']:
                 print >>sys.stderr, "Pattern '%s' for '%s' is absent in the requested content" %(ck['check'], ck['path'])
                 r |= RET_ERR
                 continue
             pr = RET_ERR
             lines = self.files[ck['path']].split('\n')
             for line in lines:
                 if ck['check'] in line:
                     pr = RET_OK
                     break
             if pr != RET_OK:
                 print "Trying to add to '%s', content: '%s'" %(ck['path'], ck['add'].strip('\n'))
                 self.files[ck['path']] += ck['add']
                 self.upds[ck['path']] = 1
             r |= pr
        r |= self.add_2_files()
        return r

    def fix(self):
        r = 0
        r != self.fix_files()
        return r


if __name__ == "__main__":
    syntax = """syntax:
      %s PREFIX check|fixable|fix"""%sys.argv[0]
    if len(sys.argv) != 3:
        print >>sys.stderr, "wrong number of arguments"
        print >>sys.stderr, syntax
        sys.exit(RET_ERR)
    try:
        o = FileInc(sys.argv[1])
        if sys.argv[2] == 'check':
            RET = o.check()
        elif sys.argv[2] == 'fix':
            RET = o.fix()
        elif sys.argv[2] == 'fixable':
            RET = o.fixable()
        else:
            print >>sys.stderr, "unsupported argument '%s'"%sys.argv[2]
            print >>sys.stderr, syntax
            RET = RET_ERR
    except ComplianceError:
        sys.exit(RET_ERR)
    except NotApplicable:
        sys.exit(RET_NA)
    except:
        import traceback
        traceback.print_exc()
        sys.exit(RET_ERR)

    sys.exit(RET)

